=== modified file 'drizzled/plugin/listen.cc'
--- drizzled/plugin/listen.cc	2011-01-24 20:14:21 +0000
+++ drizzled/plugin/listen.cc	2011-01-31 14:35:57 +0000
@@ -32,20 +32,27 @@
 
 namespace drizzled
 {
+namespace plugin
+{
 
-std::vector<plugin::Listen *> listen_list;
+static std::vector<plugin::Listen *> listen_list;
 std::vector<plugin::Listen *> listen_fd_list;
 std::vector<pollfd> fd_list;
 uint32_t fd_count= 0;
 int wakeup_pipe[2];
 
-bool plugin::Listen::addPlugin(plugin::Listen *listen_obj)
+ListenVector &Listen::getListenProtocols()
+{
+  return listen_list;
+}
+
+bool Listen::addPlugin(plugin::Listen *listen_obj)
 {
   listen_list.push_back(listen_obj);
   return false;
 }
 
-void plugin::Listen::removePlugin(plugin::Listen *listen_obj)
+void Listen::removePlugin(plugin::Listen *listen_obj)
 {
   listen_list.erase(std::remove(listen_list.begin(),
                                 listen_list.end(),
@@ -53,7 +60,7 @@
                     listen_list.end());
 }
 
-bool plugin::Listen::setup(void)
+bool Listen::setup(void)
 {
   std::vector<plugin::Listen *>::iterator it;
 
@@ -105,7 +112,7 @@
   return false;
 }
 
-plugin::Client *plugin::Listen::getClient(void)
+Client *plugin::Listen::getClient(void)
 {
   int ready;
   plugin::Client *client;
@@ -156,15 +163,17 @@
   }
 }
 
-plugin::Client *plugin::Listen::getNullClient(void)
+Client *plugin::Listen::getNullClient(void)
 {
   return new plugin::NullClient();
 }
 
-void plugin::Listen::shutdown(void)
+void Listen::shutdown(void)
 {
   ssize_t ret= write(wakeup_pipe[1], "\0", 1);
   assert(ret == 1);
 }
 
+
+} /* namespace plugin */
 } /* namespace drizzled */

=== modified file 'drizzled/plugin/listen.h'
--- drizzled/plugin/listen.h	2011-01-26 19:15:55 +0000
+++ drizzled/plugin/listen.h	2011-01-31 14:35:57 +0000
@@ -21,7 +21,7 @@
 #define DRIZZLED_PLUGIN_LISTEN_H
 
 #include "drizzled/plugin/plugin.h"
-
+#include "drizzled/atomics.h"
 #include <vector>
 
 #include "drizzled/visibility.h"
@@ -32,7 +32,9 @@
 {
 
 class Client;
-
+class Listen;
+typedef std::vector<Listen *> ListenVector;
+typedef std::pair<std::string*, drizzled::atomic<uint64_t>*> ListenCounter;
 /**
  * This class is used by client plugins to provide and manage the listening
  * interface for new client instances.
@@ -42,12 +44,20 @@
   Listen();
   Listen(const Listen&);
   Listen& operator=(const Listen&);
+protected:
+  std::vector<ListenCounter*> counters;
 public:
   explicit Listen(std::string name_arg)
     : Plugin(name_arg, "Listen")
   {}
   virtual ~Listen() {}
 
+  static ListenVector &getListenProtocols();
+
+  std::vector<ListenCounter*>& getCounters()
+  {
+    return counters;
+  }
   /**
    * This provides a list of file descriptors to watch that will trigger new
    * Client instances. When activity is detected on one of the returned file
@@ -98,6 +108,7 @@
 };
 
 } /* namespace plugin */
+
 } /* namespace drizzled */
 
 #endif /* DRIZZLED_PLUGIN_LISTEN_H */

=== modified file 'plugin/drizzle_protocol/drizzle_protocol.cc'
--- plugin/drizzle_protocol/drizzle_protocol.cc	2010-12-18 05:18:30 +0000
+++ plugin/drizzle_protocol/drizzle_protocol.cc	2011-01-31 14:35:57 +0000
@@ -98,7 +98,9 @@
   const module::option_map &vm= context.getOptions();
 
   context.add(new StatusTable);
-  context.add(new ListenDrizzleProtocol("drizzle_protocol", vm["bind-address"].as<std::string>(), true));
+  ListenDrizzleProtocol *protocol=new ListenDrizzleProtocol("drizzle_protocol", vm["bind-address"].as<std::string>(), true);
+  protocol->addCountersToTable();
+  context.add(protocol);
   context.registerVariable(new sys_var_constrained_value_readonly<in_port_t>("port", port));
   context.registerVariable(new sys_var_constrained_value_readonly<uint32_t>("connect_timeout", connect_timeout));
   context.registerVariable(new sys_var_constrained_value_readonly<uint32_t>("read_timeout", read_timeout));

=== modified file 'plugin/mysql_protocol/mysql_protocol.cc'
--- plugin/mysql_protocol/mysql_protocol.cc	2011-01-17 18:16:53 +0000
+++ plugin/mysql_protocol/mysql_protocol.cc	2011-01-31 14:35:57 +0000
@@ -66,6 +66,13 @@
 ListenMySQLProtocol::~ListenMySQLProtocol()
 { }
 
+void ListenMySQLProtocol::addCountersToTable()
+{
+  counters.push_back(new drizzled::plugin::ListenCounter(new std::string("connection_count"), &getCounters()->connectionCount));
+  counters.push_back(new drizzled::plugin::ListenCounter(new std::string("connected"), &getCounters()->connected));
+  counters.push_back(new drizzled::plugin::ListenCounter(new std::string("failed_connections"), &getCounters()->failedConnections));
+}
+
 const std::string ListenMySQLProtocol::getHost(void) const
 {
   return _hostname;
@@ -992,6 +999,7 @@
   context.add(mysql_password);
 
   listen_obj= new ListenMySQLProtocol("mysql_protocol", vm["bind-address"].as<std::string>(), true);
+  listen_obj->addCountersToTable();
   context.add(listen_obj); 
   context.registerVariable(new sys_var_constrained_value_readonly<in_port_t>("port", port));
   context.registerVariable(new sys_var_constrained_value<uint32_t>("connect_timeout", connect_timeout));

=== modified file 'plugin/mysql_protocol/mysql_protocol.h'
--- plugin/mysql_protocol/mysql_protocol.h	2010-12-18 18:24:57 +0000
+++ plugin/mysql_protocol/mysql_protocol.h	2011-01-31 14:35:57 +0000
@@ -69,6 +69,7 @@
   virtual drizzled::plugin::Client *getClient(int fd);
   static ProtocolCounters *mysql_counters;
   virtual ProtocolCounters *getCounters(void) const { return mysql_counters; }
+  void addCountersToTable(void);
 };
 
 class ClientMySQLProtocol: public drizzled::plugin::Client

=== modified file 'plugin/mysql_unix_socket_protocol/protocol.cc'
--- plugin/mysql_unix_socket_protocol/protocol.cc	2011-01-10 20:31:03 +0000
+++ plugin/mysql_unix_socket_protocol/protocol.cc	2011-01-31 14:35:57 +0000
@@ -73,6 +73,7 @@
     Protocol *listen_obj= new Protocol("mysql_unix_socket_protocol",
                              true,
                              uds_path);
+    listen_obj->addCountersToTable();
     context.add(listen_obj);
     context.registerVariable(new sys_var_const_string_val("path", fs::system_complete(uds_path).file_string()));
     context.registerVariable(new sys_var_bool_ptr_readonly("clobber", &clobber));

=== added directory 'plugin/protocol_dictionary'
=== added file 'plugin/protocol_dictionary/dictionary.cc'
--- plugin/protocol_dictionary/dictionary.cc	1970-01-01 00:00:00 +0000
+++ plugin/protocol_dictionary/dictionary.cc	2011-01-31 14:35:57 +0000
@@ -0,0 +1,98 @@
+/* - mode: c; c-basic-offset: 2; indent-tabs-mode: nil; -*-
+ *  vim:expandtab:shiftwidth=2:tabstop=2:smarttab:
+ *
+ *  Copyright (C) 2011 Andrew Hutchings
+ *
+ *  This program is free software; you can redistribute it and/or modify
+ *  it under the terms of the GNU General Public License as published by
+ *  the Free Software Foundation; either version 2 of the License, or
+ *  (at your option) any later version.
+ *
+ *  This program is distributed in the hope that it will be useful,
+ *  but WITHOUT ANY WARRANTY; without even the implied warranty of
+ *  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
+ *  GNU General Public License for more details.
+ *
+ *  You should have received a copy of the GNU General Public License
+ *  along with this program; if not, write to the Free Software
+ *  Foundation, Inc., 51 Franklin St, Fifth Floor, Boston, MA  02110-1301  USA
+ */
+
+#include "config.h"
+#include "plugin/protocol_dictionary/dictionary.h"
+#include "drizzled/plugin/listen.h"
+
+using namespace drizzled;
+
+ProtocolTool::ProtocolTool() :
+  plugin::TableFunction("DATA_DICTIONARY", "PROTOCOL_COUNTERS")
+{
+  add_field("PROTOCOL");
+  add_field("COUNTER");
+  add_field("VALUE", plugin::TableFunction::NUMBER, 0, false);
+}
+
+ProtocolTool::Generator::Generator(Field **arg) :
+  plugin::TableFunction::Generator(arg)
+{
+  protocol_it= plugin::Listen::getListenProtocols().begin();
+  protocol= *protocol_it;
+  counter_it= protocol->getCounters().begin();
+}
+
+bool ProtocolTool::Generator::populate()
+{
+  if (protocol_it == plugin::Listen::getListenProtocols().end())
+    return false;
+
+  protocol= *protocol_it;
+
+  while (counter_it == protocol->getCounters().end())
+  {
+    protocol_it++;
+    if (protocol_it == plugin::Listen::getListenProtocols().end())
+      return false;
+    protocol= *protocol_it;
+    counter_it= protocol->getCounters().begin();
+  }
+
+  fill();
+  counter_it++;
+  return true;
+}
+
+void ProtocolTool::Generator::fill()
+{
+  protocol= *protocol_it;
+  counter= *counter_it;
+
+  push(protocol->getName());
+  push(*counter->first);
+  push((uint64_t) *counter->second);
+}
+
+static ProtocolTool *protocols;
+
+static int init(drizzled::module::Context &context)
+{
+  protocols= new(std::nothrow)ProtocolTool;
+
+  context.add(protocols);
+
+  return 0;
+}
+
+DRIZZLE_DECLARE_PLUGIN
+{
+  DRIZZLE_VERSION_ID,
+  "protocol_dictionary",
+  "1.0",
+  "Andrew Hutchings",
+  "Provides dictionary for protocol counters.",
+  PLUGIN_LICENSE_GPL,
+  init,     /* Plugin Init */
+  NULL,               /* depends */
+  NULL                /* config options   */
+}
+DRIZZLE_DECLARE_PLUGIN_END;
+

=== added file 'plugin/protocol_dictionary/dictionary.h'
--- plugin/protocol_dictionary/dictionary.h	1970-01-01 00:00:00 +0000
+++ plugin/protocol_dictionary/dictionary.h	2011-01-31 14:35:57 +0000
@@ -0,0 +1,50 @@
+/* - mode: c; c-basic-offset: 2; indent-tabs-mode: nil; -*-
+ *  vim:expandtab:shiftwidth=2:tabstop=2:smarttab:
+ *
+ *  Copyright (C) 2011 Andrew Hutchings
+ *
+ *  This program is free software; you can redistribute it and/or modify
+ *  it under the terms of the GNU General Public License as published by
+ *  the Free Software Foundation; either version 2 of the License, or
+ *  (at your option) any later version.
+ *
+ *  This program is distributed in the hope that it will be useful,
+ *  but WITHOUT ANY WARRANTY; without even the implied warranty of
+ *  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
+ *  GNU General Public License for more details.
+ *
+ *  You should have received a copy of the GNU General Public License
+ *  along with this program; if not, write to the Free Software
+ *  Foundation, Inc., 51 Franklin St, Fifth Floor, Boston, MA  02110-1301  USA
+ */
+
+#ifndef PLUGIN_PROTOCOL_DICTIONARY_DICTIONARY_H
+#define PLUGIN_PROTOCOL_DICTIONARY_DICTIONARY_H
+
+#include "drizzled/plugin/table_function.h"
+#include "drizzled/plugin/listen.h"
+
+class ProtocolTool : public drizzled::plugin::TableFunction
+{
+public:
+
+  ProtocolTool();
+
+  class Generator : public drizzled::plugin::TableFunction::Generator
+  {
+    std::vector<drizzled::plugin::Listen *>::iterator protocol_it;
+    std::vector<drizzled::plugin::ListenCounter *>::iterator counter_it;
+    drizzled::plugin::Listen *protocol;
+    drizzled::plugin::ListenCounter *counter;
+  public:
+    Generator(drizzled::Field **arg);
+    void fill();
+    bool populate();
+  };
+
+  Generator *generator(drizzled::Field **arg)
+  {
+    return new Generator(arg);
+  }
+};
+#endif /* PLUGIN_PROTOCOL_DICTIONARY_DICTIONARY_H */

=== added file 'plugin/protocol_dictionary/plugin.ini'
--- plugin/protocol_dictionary/plugin.ini	1970-01-01 00:00:00 +0000
+++ plugin/protocol_dictionary/plugin.ini	2011-01-31 14:35:57 +0000
@@ -0,0 +1,9 @@
+[plugin]
+title=Protocol Dictionary
+description=Provides dictionary tables for protocol counters
+load_by_default=yes
+sources=
+  dictionary.cc
+headers=
+  dictionary.h
+static=yes

